from setuptools import setup

setup(name="spout",
      version="0.1.0",
      description="A simple framework that makes it easy to work with data streams in Python.",
      url="http://github.com/daviesjamie/spout",
      author="Jamie Davies",
      author_email="jamie@jamiedavies.me",
      license="MIT",
      packages=['spout'],
      zip_safe=False)
